// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __LOBBY_H
#define __LOBBY_H

#include "select.h"
#include "connectionmanager.h"

#include <string>
#include <list>

class ConnectionManager;
class TunerManager;
class Scheduler;
class UnixServerSocket;

class Lobby : public Selectable {
public:
  Lobby(const std::string &socketFilename, 
	const std::string &socketUid,
	const std::string &socketGid,
	const std::string &socketMode,
	TunerManager *tm,
	ConnectionManager *cm,
	Scheduler *scheduler);

  virtual ~Lobby();

  virtual void addSelectFDs(Select &s) const;
  virtual bool isReady(const Select &s) const;
  void processReady(Select &s);

private:
  UnixServerSocket *server;
  TunerManager *tm;
  ConnectionManager *cm;
  Scheduler *scheduler;

  void setRandomSeed();
  std::string generateId();

  // Associates the file descriptor
  // with it's id (cookie)
  class Control {
  public:
    Control(int fd, const std::string &id)
      : fd(fd), id(id)
    {
    };

    int getFD() const {
      return fd;
    }

    bool matches(const std::string &id) {
      if (this->id.empty())
	return false;
      else 
	return this->id == id;
    }

    void setId(const std::string &id) {
      this->id = id;
    };

    const std::string &getId() const {
      return id;
    }

  private:
    int fd;
    std::string id;
  };

  typedef std::list<int> ClientList;
  ClientList clients;

  typedef std::list<Control> ControlList;
  ControlList controllers;
};

#endif // __LOBBY_H

