// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __TUNERMANAGER_H
#define __TUNERMANAGER_H

#include "dvbd.h"
#include "select.h"
#include "source.h"

#include <list>
#include <map>
#include <string>

class Tuner;
class ConfigFile;

typedef std::list<Tuner *> TunerList;

class TunerManager : public Selectable {
public:

  virtual ~TunerManager();

  bool TunerManager::parseConfig(ConfigFile &c);
  void parseCommandLine(int argc, char *argv[]);

  virtual void addSelectFDs(Select &) const;
  virtual bool isReady(const Select &) const;
  void processReady(const Select &s);

  bool add( const std::string &type, 
	    const std::string &device, 
	    const std::string &channels );

  void getDemuxers(const std::string &type, const std::string &channel,
		   bool convertToPS, SourceList &demuxers, int priority);

  bool canGetDemuxers(const std::string &type, const std::string &channel,
		      int priority) const;

  bool validType(const std::string &type) const;
  bool validChannel(const std::string &type, const std::string &channel) const;

  void getTunableChannels(const std::string &type, int priority, StringList &channels) const;

private:
  friend class FakeTunerManager;

  typedef std::map<Tuner *, ChannelMap *> TunerChannelMap;
  typedef std::map<std::string, TunerList *> TypeTunerMap;

  TunerList tuners;
  TunerChannelMap tunerChannelMap;
  TypeTunerMap typeTunerMap;
};

#endif // __TUNERMANAGER_H
